import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.attachment_order_payment import AttachmentOrderPayment
from ..models.attachment_order_type import AttachmentOrderType
from ..models.bank_details import BankDetails
from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="AttachmentOrder")


@attr.s(auto_attribs=True)
class AttachmentOrder:
    """Used to represent an Attachment of Earnings Order (AEO)

    Attributes:
        type (Union[Unset, AttachmentOrderType]):
        carry_forward_shortfall (Union[Unset, bool]): [ReadOnly] Whether or not shortfalls should be carried forward
        allow_protected_earnings (Union[Unset, bool]): [ReadOnly] Whether or not shortfalls should be carried forward
        auto_deduction (Union[Unset, bool]): [readonly] Whether or not the amount to be deducted is automatically
            calculated
        reference (Union[Unset, None, str]): The reference which appeared on the court paperwork
        issue_date (Union[Unset, datetime.date]): The date of issue for this Order
        apply_from (Union[Unset, datetime.date]): The date from which to apply this Order
        apply_until (Union[Unset, None, datetime.date]): An optional date on which to stop applying this order
        deduction_is_percentage (Union[Unset, bool]): Whether or not the amount given for Deduction is a percentage
            rather than a fixed amount
        deduction (Union[Unset, float]): The percentage or amount (depending on DeductionIsPercentage) to deduct
        protected_earnings_is_percentage (Union[Unset, bool]): Whether or not the amount for ProtectedEarnings is a
            percentage rather than a fixed amount.
        protected_earnings (Union[Unset, float]): The percentage or amount or percentage (depending on
            ProtectedEarningsIsPercentage) to protect
        charge_admin_fee (Union[Unset, bool]): Whether or not a £1 admin fee should be deducted for this order
        shortfall (Union[Unset, float]): Any shortfall that is being carried forward
        stop_when_total_paid (Union[Unset, bool]): Whether or not this order should be stopped once a total amount has
            been paid
        total_amount_to_pay (Union[Unset, float]): The Total amount that needs to be paid for this Order
        amount_previously_paid (Union[Unset, float]): Any amount that has previously been paid towards this Order
        stopped (Union[Unset, bool]): [readonly] Whether or not this Order has been stopped.
            This is set automatically when either it's paid in full or the ApplyUntil date has been reached.
        notes (Union[Unset, None, str]): A free-form text field to record any comments
        include_bank_details (Union[Unset, bool]):
        bank_details (Union[Unset, BankDetails]):
        payments (Union[Unset, None, List[AttachmentOrderPayment]]): [readonly] Payments made towards this order. Only
            populated when viewed as a report.
        payee (Union[Unset, None, str]): The Id of the Payee, if any, that deductions are to be paid to.
        document_count (Union[Unset, int]): [readonly] The number of attachments associated with this model
        documents (Union[Unset, None, List[Item]]): [readonly] The attachments associated with this model
        employee (Union[Unset, Item]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    type: Union[Unset, AttachmentOrderType] = UNSET
    carry_forward_shortfall: Union[Unset, bool] = UNSET
    allow_protected_earnings: Union[Unset, bool] = UNSET
    auto_deduction: Union[Unset, bool] = UNSET
    reference: Union[Unset, None, str] = UNSET
    issue_date: Union[Unset, datetime.date] = UNSET
    apply_from: Union[Unset, datetime.date] = UNSET
    apply_until: Union[Unset, None, datetime.date] = UNSET
    deduction_is_percentage: Union[Unset, bool] = UNSET
    deduction: Union[Unset, float] = UNSET
    protected_earnings_is_percentage: Union[Unset, bool] = UNSET
    protected_earnings: Union[Unset, float] = UNSET
    charge_admin_fee: Union[Unset, bool] = UNSET
    shortfall: Union[Unset, float] = UNSET
    stop_when_total_paid: Union[Unset, bool] = UNSET
    total_amount_to_pay: Union[Unset, float] = UNSET
    amount_previously_paid: Union[Unset, float] = UNSET
    stopped: Union[Unset, bool] = UNSET
    notes: Union[Unset, None, str] = UNSET
    include_bank_details: Union[Unset, bool] = UNSET
    bank_details: Union[Unset, BankDetails] = UNSET
    payments: Union[Unset, None, List[AttachmentOrderPayment]] = UNSET
    payee: Union[Unset, None, str] = UNSET
    document_count: Union[Unset, int] = UNSET
    documents: Union[Unset, None, List[Item]] = UNSET
    employee: Union[Unset, Item] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        carry_forward_shortfall = self.carry_forward_shortfall
        allow_protected_earnings = self.allow_protected_earnings
        auto_deduction = self.auto_deduction
        reference = self.reference
        issue_date: Union[Unset, str] = UNSET
        if not isinstance(self.issue_date, Unset):
            issue_date = self.issue_date.isoformat()

        apply_from: Union[Unset, str] = UNSET
        if not isinstance(self.apply_from, Unset):
            apply_from = self.apply_from.isoformat()

        apply_until: Union[Unset, None, str] = UNSET
        if not isinstance(self.apply_until, Unset):
            apply_until = self.apply_until.isoformat() if self.apply_until else None

        deduction_is_percentage = self.deduction_is_percentage
        deduction = self.deduction
        protected_earnings_is_percentage = self.protected_earnings_is_percentage
        protected_earnings = self.protected_earnings
        charge_admin_fee = self.charge_admin_fee
        shortfall = self.shortfall
        stop_when_total_paid = self.stop_when_total_paid
        total_amount_to_pay = self.total_amount_to_pay
        amount_previously_paid = self.amount_previously_paid
        stopped = self.stopped
        notes = self.notes
        include_bank_details = self.include_bank_details
        bank_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.bank_details, Unset):
            bank_details = self.bank_details.to_dict()

        payments: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.payments, Unset):
            if self.payments is None:
                payments = None
            else:
                payments = []
                for payments_item_data in self.payments:
                    payments_item = payments_item_data.to_dict()

                    payments.append(payments_item)

        payee = self.payee
        document_count = self.document_count
        documents: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.documents, Unset):
            if self.documents is None:
                documents = None
            else:
                documents = []
                for documents_item_data in self.documents:
                    documents_item = documents_item_data.to_dict()

                    documents.append(documents_item)

        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if carry_forward_shortfall is not UNSET:
            field_dict["carryForwardShortfall"] = carry_forward_shortfall
        if allow_protected_earnings is not UNSET:
            field_dict["allowProtectedEarnings"] = allow_protected_earnings
        if auto_deduction is not UNSET:
            field_dict["autoDeduction"] = auto_deduction
        if reference is not UNSET:
            field_dict["reference"] = reference
        if issue_date is not UNSET:
            field_dict["issueDate"] = issue_date
        if apply_from is not UNSET:
            field_dict["applyFrom"] = apply_from
        if apply_until is not UNSET:
            field_dict["applyUntil"] = apply_until
        if deduction_is_percentage is not UNSET:
            field_dict["deductionIsPercentage"] = deduction_is_percentage
        if deduction is not UNSET:
            field_dict["deduction"] = deduction
        if protected_earnings_is_percentage is not UNSET:
            field_dict[
                "protectedEarningsIsPercentage"
            ] = protected_earnings_is_percentage
        if protected_earnings is not UNSET:
            field_dict["protectedEarnings"] = protected_earnings
        if charge_admin_fee is not UNSET:
            field_dict["chargeAdminFee"] = charge_admin_fee
        if shortfall is not UNSET:
            field_dict["shortfall"] = shortfall
        if stop_when_total_paid is not UNSET:
            field_dict["stopWhenTotalPaid"] = stop_when_total_paid
        if total_amount_to_pay is not UNSET:
            field_dict["totalAmountToPay"] = total_amount_to_pay
        if amount_previously_paid is not UNSET:
            field_dict["amountPreviouslyPaid"] = amount_previously_paid
        if stopped is not UNSET:
            field_dict["stopped"] = stopped
        if notes is not UNSET:
            field_dict["notes"] = notes
        if include_bank_details is not UNSET:
            field_dict["includeBankDetails"] = include_bank_details
        if bank_details is not UNSET:
            field_dict["bankDetails"] = bank_details
        if payments is not UNSET:
            field_dict["payments"] = payments
        if payee is not UNSET:
            field_dict["payee"] = payee
        if document_count is not UNSET:
            field_dict["documentCount"] = document_count
        if documents is not UNSET:
            field_dict["documents"] = documents
        if employee is not UNSET:
            field_dict["employee"] = employee
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, AttachmentOrderType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = AttachmentOrderType(_type)

        carry_forward_shortfall = d.pop("carryForwardShortfall", UNSET)

        allow_protected_earnings = d.pop("allowProtectedEarnings", UNSET)

        auto_deduction = d.pop("autoDeduction", UNSET)

        reference = d.pop("reference", UNSET)

        _issue_date = d.pop("issueDate", UNSET)
        issue_date: Union[Unset, datetime.date]
        if isinstance(_issue_date, Unset):
            issue_date = UNSET
        else:
            issue_date = isoparse(_issue_date).date()

        _apply_from = d.pop("applyFrom", UNSET)
        apply_from: Union[Unset, datetime.date]
        if isinstance(_apply_from, Unset):
            apply_from = UNSET
        else:
            apply_from = isoparse(_apply_from).date()

        _apply_until = d.pop("applyUntil", UNSET)
        apply_until: Union[Unset, None, datetime.date]
        if _apply_until is None:
            apply_until = None
        elif isinstance(_apply_until, Unset):
            apply_until = UNSET
        else:
            apply_until = isoparse(_apply_until).date()

        deduction_is_percentage = d.pop("deductionIsPercentage", UNSET)

        deduction = d.pop("deduction", UNSET)

        protected_earnings_is_percentage = d.pop("protectedEarningsIsPercentage", UNSET)

        protected_earnings = d.pop("protectedEarnings", UNSET)

        charge_admin_fee = d.pop("chargeAdminFee", UNSET)

        shortfall = d.pop("shortfall", UNSET)

        stop_when_total_paid = d.pop("stopWhenTotalPaid", UNSET)

        total_amount_to_pay = d.pop("totalAmountToPay", UNSET)

        amount_previously_paid = d.pop("amountPreviouslyPaid", UNSET)

        stopped = d.pop("stopped", UNSET)

        notes = d.pop("notes", UNSET)

        include_bank_details = d.pop("includeBankDetails", UNSET)

        _bank_details = d.pop("bankDetails", UNSET)
        bank_details: Union[Unset, BankDetails]
        if isinstance(_bank_details, Unset):
            bank_details = UNSET
        else:
            bank_details = BankDetails.from_dict(_bank_details)

        payments = []
        _payments = d.pop("payments", UNSET)
        for payments_item_data in _payments or []:
            payments_item = AttachmentOrderPayment.from_dict(payments_item_data)

            payments.append(payments_item)

        payee = d.pop("payee", UNSET)

        document_count = d.pop("documentCount", UNSET)

        documents = []
        _documents = d.pop("documents", UNSET)
        for documents_item_data in _documents or []:
            documents_item = Item.from_dict(documents_item_data)

            documents.append(documents_item)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        id = d.pop("id", UNSET)

        attachment_order = cls(
            type=type,
            carry_forward_shortfall=carry_forward_shortfall,
            allow_protected_earnings=allow_protected_earnings,
            auto_deduction=auto_deduction,
            reference=reference,
            issue_date=issue_date,
            apply_from=apply_from,
            apply_until=apply_until,
            deduction_is_percentage=deduction_is_percentage,
            deduction=deduction,
            protected_earnings_is_percentage=protected_earnings_is_percentage,
            protected_earnings=protected_earnings,
            charge_admin_fee=charge_admin_fee,
            shortfall=shortfall,
            stop_when_total_paid=stop_when_total_paid,
            total_amount_to_pay=total_amount_to_pay,
            amount_previously_paid=amount_previously_paid,
            stopped=stopped,
            notes=notes,
            include_bank_details=include_bank_details,
            bank_details=bank_details,
            payments=payments,
            payee=payee,
            document_count=document_count,
            documents=documents,
            employee=employee,
            id=id,
        )

        return attachment_order



from enum import Enum


class ExternalDataProviderId(str, Enum):
    SMARTPENSION_EMPLOYER = "SmartPension_Employer"
    SMARTPENSION_ADVISER = "SmartPension_Adviser"
    PEOPLESPENSION = "PeoplesPension"
    XERO = "Xero"
    QBO = "Qbo"
    SAGE = "Sage"
    NEST = "Nest"
    CASHPLUS = "Cashplus"
    BREATHEHR = "BreatheHr"
    KASHFLOW = "KashFlow"
    RECKON = "Reckon"
    FREEAGENT = "FreeAgent"
    FREEAGENTPM = "FreeAgentPm"
    MODULR = "Modulr"
    SQUARE = "Square"
    CEZANNE = "Cezanne"
    CIPHR = "Ciphr"
    TELLEROO = "Telleroo"
    WEWORKED = "WeWorked"
    ROTACLOUD = "RotaCloud"
    QUINYX = "Quinyx"
    CSOD = "Csod"
    PLANDAY = "Planday"
    STARLING = "Starling"
    DEPUTY = "Deputy"
    BOTTOMLINE = "Bottomline"
    TWINFIELD = "Twinfield"
    MYEPAYWINDOW = "MyePayWindow"
    IFINANCE = "IFinance"
    ACCOUNTSIQ = "AccountsIQ"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class EmployerTemplateType(str, Enum):
    PAYSLIPEMAIL = "PayslipEmail"
    AUTOENROLMENT_ENROLLED = "AutoEnrolment_Enrolled"
    AUTOENROLMENT_ENROLLED_NETPAY = "AutoEnrolment_Enrolled_NetPay"
    AUTOENROLMENT_NOTENROLLED = "AutoEnrolment_NotEnrolled"
    AUTOENROLMENT_INSERT = "AutoEnrolment_Insert"
    CISSTATEMENTEMAIL = "CisStatementEmail"
    PAYRUNSUMMARY = "PayrunSummary"
    PAYSLIPSUNEMAILED = "PayslipsUnemailed"
    PAYRUNAUTOEMAIL = "PayrunAutoEmail"
    P60EMAIL = "P60Email"
    ANNUALCISSTATEMENTEMAIL = "AnnualCisStatementEmail"
    P45EMAIL = "P45Email"
    AUTOENROLMENT_POSTPONED = "AutoEnrolment_Postponed"
    AUTOENROLMENT_REENROLLED = "AutoEnrolment_ReEnrolled"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Optional, Union

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.external_data_provider_id import ExternalDataProviderId
from ...models.pay_options_import import PayOptionsImport
from ...models.pay_periods import PayPeriods
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    pay_period: PayPeriods,
    *,
    client: Client,
    provider_id: Union[Unset, None, ExternalDataProviderId] = UNSET,
    ordinal: Union[Unset, None, int] = 1,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/payrun/{payPeriod}/importtimeandattendance".format(
        client.base_url, employerId=employer_id, payPeriod=pay_period
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_provider_id: Union[Unset, None, str] = UNSET
    if not isinstance(provider_id, Unset):
        json_provider_id = provider_id.value if provider_id else None

    params["providerId"] = json_provider_id

    params["ordinal"] = ordinal

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[PayOptionsImport]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = PayOptionsImport.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    return raise_staffology_exception(response)


def _build_response(*, response: httpx.Response) -> Response[List[PayOptionsImport]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    pay_period: PayPeriods,
    *,
    client: Client,
    provider_id: Union[Unset, None, ExternalDataProviderId] = UNSET,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[List[PayOptionsImport]]:
    """Import Time And Attendance

     If the employer is connected to an ExternalDataProvider that provides Time and Attendance data then
    this API
    call will update the currently open payrun for the PayPeriod specified with data from the specified
    provider.
    Returns a list of PayOptionsImport to show what's been imported.

    Args:
        employer_id (str):
        pay_period (PayPeriods):
        provider_id (Union[Unset, None, ExternalDataProviderId]):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[List[PayOptionsImport]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        pay_period=pay_period,
        client=client,
        provider_id=provider_id,
        ordinal=ordinal,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    pay_period: PayPeriods,
    *,
    client: Client,
    provider_id: Union[Unset, None, ExternalDataProviderId] = UNSET,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[List[PayOptionsImport]]:
    """Import Time And Attendance

     If the employer is connected to an ExternalDataProvider that provides Time and Attendance data then
    this API
    call will update the currently open payrun for the PayPeriod specified with data from the specified
    provider.
    Returns a list of PayOptionsImport to show what's been imported.

    Args:
        employer_id (str):
        pay_period (PayPeriods):
        provider_id (Union[Unset, None, ExternalDataProviderId]):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[List[PayOptionsImport]]
    """

    return sync_detailed(
        employer_id=employer_id,
        pay_period=pay_period,
        client=client,
        provider_id=provider_id,
        ordinal=ordinal,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    pay_period: PayPeriods,
    *,
    client: Client,
    provider_id: Union[Unset, None, ExternalDataProviderId] = UNSET,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[List[PayOptionsImport]]:
    """Import Time And Attendance

     If the employer is connected to an ExternalDataProvider that provides Time and Attendance data then
    this API
    call will update the currently open payrun for the PayPeriod specified with data from the specified
    provider.
    Returns a list of PayOptionsImport to show what's been imported.

    Args:
        employer_id (str):
        pay_period (PayPeriods):
        provider_id (Union[Unset, None, ExternalDataProviderId]):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[List[PayOptionsImport]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        pay_period=pay_period,
        client=client,
        provider_id=provider_id,
        ordinal=ordinal,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    pay_period: PayPeriods,
    *,
    client: Client,
    provider_id: Union[Unset, None, ExternalDataProviderId] = UNSET,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[List[PayOptionsImport]]:
    """Import Time And Attendance

     If the employer is connected to an ExternalDataProvider that provides Time and Attendance data then
    this API
    call will update the currently open payrun for the PayPeriod specified with data from the specified
    provider.
    Returns a list of PayOptionsImport to show what's been imported.

    Args:
        employer_id (str):
        pay_period (PayPeriods):
        provider_id (Union[Unset, None, ExternalDataProviderId]):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[List[PayOptionsImport]]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            pay_period=pay_period,
            client=client,
            provider_id=provider_id,
            ordinal=ordinal,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="TenantBillingSettings")


@attr.s(auto_attribs=True)
class TenantBillingSettings:
    """
    Attributes:
        discount (Union[Unset, float]):
        monthly_minimum (Union[Unset, float]):
        aggregated_pricing (Union[Unset, bool]):
        bill_to (Union[Unset, None, str]): If all activity for a Tenant is being biulled to a specifc user, set the
            email address here
    """

    discount: Union[Unset, float] = UNSET
    monthly_minimum: Union[Unset, float] = UNSET
    aggregated_pricing: Union[Unset, bool] = UNSET
    bill_to: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        discount = self.discount
        monthly_minimum = self.monthly_minimum
        aggregated_pricing = self.aggregated_pricing
        bill_to = self.bill_to

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if discount is not UNSET:
            field_dict["discount"] = discount
        if monthly_minimum is not UNSET:
            field_dict["monthlyMinimum"] = monthly_minimum
        if aggregated_pricing is not UNSET:
            field_dict["aggregatedPricing"] = aggregated_pricing
        if bill_to is not UNSET:
            field_dict["billTo"] = bill_to

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        discount = d.pop("discount", UNSET)

        monthly_minimum = d.pop("monthlyMinimum", UNSET)

        aggregated_pricing = d.pop("aggregatedPricing", UNSET)

        bill_to = d.pop("billTo", UNSET)

        tenant_billing_settings = cls(
            discount=discount,
            monthly_minimum=monthly_minimum,
            aggregated_pricing=aggregated_pricing,
            bill_to=bill_to,
        )

        return tenant_billing_settings



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="GrossToNetReportCisLine")


@attr.s(auto_attribs=True)
class GrossToNetReportCisLine:
    """
    Attributes:
        employee (Union[Unset, Item]):
        payroll_code (Union[Unset, None, str]):
        department (Union[Unset, None, str]):
        total_gross (Union[Unset, float]):
        labour (Union[Unset, float]):
        materials (Union[Unset, float]):
        taxable_gross (Union[Unset, float]):
        cis_deduction (Union[Unset, float]):
        umbrella_fee (Union[Unset, float]):
        vat (Union[Unset, float]):
        payment (Union[Unset, float]):
    """

    employee: Union[Unset, Item] = UNSET
    payroll_code: Union[Unset, None, str] = UNSET
    department: Union[Unset, None, str] = UNSET
    total_gross: Union[Unset, float] = UNSET
    labour: Union[Unset, float] = UNSET
    materials: Union[Unset, float] = UNSET
    taxable_gross: Union[Unset, float] = UNSET
    cis_deduction: Union[Unset, float] = UNSET
    umbrella_fee: Union[Unset, float] = UNSET
    vat: Union[Unset, float] = UNSET
    payment: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        payroll_code = self.payroll_code
        department = self.department
        total_gross = self.total_gross
        labour = self.labour
        materials = self.materials
        taxable_gross = self.taxable_gross
        cis_deduction = self.cis_deduction
        umbrella_fee = self.umbrella_fee
        vat = self.vat
        payment = self.payment

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee is not UNSET:
            field_dict["employee"] = employee
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if department is not UNSET:
            field_dict["department"] = department
        if total_gross is not UNSET:
            field_dict["totalGross"] = total_gross
        if labour is not UNSET:
            field_dict["labour"] = labour
        if materials is not UNSET:
            field_dict["materials"] = materials
        if taxable_gross is not UNSET:
            field_dict["taxableGross"] = taxable_gross
        if cis_deduction is not UNSET:
            field_dict["cisDeduction"] = cis_deduction
        if umbrella_fee is not UNSET:
            field_dict["umbrellaFee"] = umbrella_fee
        if vat is not UNSET:
            field_dict["vat"] = vat
        if payment is not UNSET:
            field_dict["payment"] = payment

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        payroll_code = d.pop("payrollCode", UNSET)

        department = d.pop("department", UNSET)

        total_gross = d.pop("totalGross", UNSET)

        labour = d.pop("labour", UNSET)

        materials = d.pop("materials", UNSET)

        taxable_gross = d.pop("taxableGross", UNSET)

        cis_deduction = d.pop("cisDeduction", UNSET)

        umbrella_fee = d.pop("umbrellaFee", UNSET)

        vat = d.pop("vat", UNSET)

        payment = d.pop("payment", UNSET)

        gross_to_net_report_cis_line = cls(
            employee=employee,
            payroll_code=payroll_code,
            department=department,
            total_gross=total_gross,
            labour=labour,
            materials=materials,
            taxable_gross=taxable_gross,
            cis_deduction=cis_deduction,
            umbrella_fee=umbrella_fee,
            vat=vat,
            payment=payment,
        )

        return gross_to_net_report_cis_line



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Tag")


@attr.s(auto_attribs=True)
class Tag:
    """
    Attributes:
        code (str): The unique code for this Tag
        title (str): The title for this Tag
        color (Union[Unset, None, str]): A color to used to represent this Tag, in hex format. ie 'ff0000'
    """

    code: str
    title: str
    color: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        title = self.title
        color = self.color

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "code": code,
                "title": title,
            }
        )
        if color is not UNSET:
            field_dict["color"] = color

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        code = d.pop("code")

        title = d.pop("title")

        color = d.pop("color", UNSET)

        tag = cls(
            code=code,
            title=title,
            color=color,
        )

        return tag



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.benefit_declaration_type import BenefitDeclarationType
from ..models.benefit_details_asset_type import BenefitDetailsAssetType
from ..models.benefit_details_car import BenefitDetailsCar
from ..models.benefit_details_class_1a_type import BenefitDetailsClass1AType
from ..models.benefit_details_loan import BenefitDetailsLoan
from ..models.benefit_details_non_class_1a_type import BenefitDetailsNonClass1AType
from ..models.benefit_details_payment_type import BenefitDetailsPaymentType
from ..models.benefit_details_use_of_asset_type import BenefitDetailsUseOfAssetType
from ..models.benefit_type import BenefitType
from ..models.item import Item
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="Benefit")


@attr.s(auto_attribs=True)
class Benefit:
    """Used to represent Benefits and Expenses

    Attributes:
        tax_year (Union[Unset, TaxYear]):
        type (Union[Unset, BenefitType]):
        declaration_type (Union[Unset, BenefitDeclarationType]):
        description (Union[Unset, None, str]): A description of this benefit
        value (Union[Unset, float]):
        employee_contribution (Union[Unset, float]):
        cash_equivalent (Union[Unset, float]): [readonly]
        asset_type (Union[Unset, BenefitDetailsAssetType]):
        use_of_asset_type (Union[Unset, BenefitDetailsUseOfAssetType]):
        class_1a_type (Union[Unset, BenefitDetailsClass1AType]):
        non_class_1a_type (Union[Unset, BenefitDetailsNonClass1AType]):
        payment_type (Union[Unset, BenefitDetailsPaymentType]):
        trading_organisation (Union[Unset, bool]): Only relevant to Benefits with Type Entertainment
        cash_equivalent_fuel (Union[Unset, float]): Only relevant to Benefits with Type Vans
        loan (Union[Unset, BenefitDetailsLoan]):
        car (Union[Unset, BenefitDetailsCar]):
        employee (Union[Unset, Item]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    tax_year: Union[Unset, TaxYear] = UNSET
    type: Union[Unset, BenefitType] = UNSET
    declaration_type: Union[Unset, BenefitDeclarationType] = UNSET
    description: Union[Unset, None, str] = UNSET
    value: Union[Unset, float] = UNSET
    employee_contribution: Union[Unset, float] = UNSET
    cash_equivalent: Union[Unset, float] = UNSET
    asset_type: Union[Unset, BenefitDetailsAssetType] = UNSET
    use_of_asset_type: Union[Unset, BenefitDetailsUseOfAssetType] = UNSET
    class_1a_type: Union[Unset, BenefitDetailsClass1AType] = UNSET
    non_class_1a_type: Union[Unset, BenefitDetailsNonClass1AType] = UNSET
    payment_type: Union[Unset, BenefitDetailsPaymentType] = UNSET
    trading_organisation: Union[Unset, bool] = UNSET
    cash_equivalent_fuel: Union[Unset, float] = UNSET
    loan: Union[Unset, BenefitDetailsLoan] = UNSET
    car: Union[Unset, BenefitDetailsCar] = UNSET
    employee: Union[Unset, Item] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        declaration_type: Union[Unset, str] = UNSET
        if not isinstance(self.declaration_type, Unset):
            declaration_type = self.declaration_type.value

        description = self.description
        value = self.value
        employee_contribution = self.employee_contribution
        cash_equivalent = self.cash_equivalent
        asset_type: Union[Unset, str] = UNSET
        if not isinstance(self.asset_type, Unset):
            asset_type = self.asset_type.value

        use_of_asset_type: Union[Unset, str] = UNSET
        if not isinstance(self.use_of_asset_type, Unset):
            use_of_asset_type = self.use_of_asset_type.value

        class_1a_type: Union[Unset, str] = UNSET
        if not isinstance(self.class_1a_type, Unset):
            class_1a_type = self.class_1a_type.value

        non_class_1a_type: Union[Unset, str] = UNSET
        if not isinstance(self.non_class_1a_type, Unset):
            non_class_1a_type = self.non_class_1a_type.value

        payment_type: Union[Unset, str] = UNSET
        if not isinstance(self.payment_type, Unset):
            payment_type = self.payment_type.value

        trading_organisation = self.trading_organisation
        cash_equivalent_fuel = self.cash_equivalent_fuel
        loan: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.loan, Unset):
            loan = self.loan.to_dict()

        car: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.car, Unset):
            car = self.car.to_dict()

        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if type is not UNSET:
            field_dict["type"] = type
        if declaration_type is not UNSET:
            field_dict["declarationType"] = declaration_type
        if description is not UNSET:
            field_dict["description"] = description
        if value is not UNSET:
            field_dict["value"] = value
        if employee_contribution is not UNSET:
            field_dict["employeeContribution"] = employee_contribution
        if cash_equivalent is not UNSET:
            field_dict["cashEquivalent"] = cash_equivalent
        if asset_type is not UNSET:
            field_dict["assetType"] = asset_type
        if use_of_asset_type is not UNSET:
            field_dict["useOfAssetType"] = use_of_asset_type
        if class_1a_type is not UNSET:
            field_dict["class1AType"] = class_1a_type
        if non_class_1a_type is not UNSET:
            field_dict["nonClass1AType"] = non_class_1a_type
        if payment_type is not UNSET:
            field_dict["paymentType"] = payment_type
        if trading_organisation is not UNSET:
            field_dict["tradingOrganisation"] = trading_organisation
        if cash_equivalent_fuel is not UNSET:
            field_dict["cashEquivalentFuel"] = cash_equivalent_fuel
        if loan is not UNSET:
            field_dict["loan"] = loan
        if car is not UNSET:
            field_dict["car"] = car
        if employee is not UNSET:
            field_dict["employee"] = employee
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        _type = d.pop("type", UNSET)
        type: Union[Unset, BenefitType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = BenefitType(_type)

        _declaration_type = d.pop("declarationType", UNSET)
        declaration_type: Union[Unset, BenefitDeclarationType]
        if isinstance(_declaration_type, Unset):
            declaration_type = UNSET
        else:
            declaration_type = BenefitDeclarationType(_declaration_type)

        description = d.pop("description", UNSET)

        value = d.pop("value", UNSET)

        employee_contribution = d.pop("employeeContribution", UNSET)

        cash_equivalent = d.pop("cashEquivalent", UNSET)

        _asset_type = d.pop("assetType", UNSET)
        asset_type: Union[Unset, BenefitDetailsAssetType]
        if isinstance(_asset_type, Unset):
            asset_type = UNSET
        else:
            asset_type = BenefitDetailsAssetType(_asset_type)

        _use_of_asset_type = d.pop("useOfAssetType", UNSET)
        use_of_asset_type: Union[Unset, BenefitDetailsUseOfAssetType]
        if isinstance(_use_of_asset_type, Unset):
            use_of_asset_type = UNSET
        else:
            use_of_asset_type = BenefitDetailsUseOfAssetType(_use_of_asset_type)

        _class_1a_type = d.pop("class1AType", UNSET)
        class_1a_type: Union[Unset, BenefitDetailsClass1AType]
        if isinstance(_class_1a_type, Unset):
            class_1a_type = UNSET
        else:
            class_1a_type = BenefitDetailsClass1AType(_class_1a_type)

        _non_class_1a_type = d.pop("nonClass1AType", UNSET)
        non_class_1a_type: Union[Unset, BenefitDetailsNonClass1AType]
        if isinstance(_non_class_1a_type, Unset):
            non_class_1a_type = UNSET
        else:
            non_class_1a_type = BenefitDetailsNonClass1AType(_non_class_1a_type)

        _payment_type = d.pop("paymentType", UNSET)
        payment_type: Union[Unset, BenefitDetailsPaymentType]
        if isinstance(_payment_type, Unset):
            payment_type = UNSET
        else:
            payment_type = BenefitDetailsPaymentType(_payment_type)

        trading_organisation = d.pop("tradingOrganisation", UNSET)

        cash_equivalent_fuel = d.pop("cashEquivalentFuel", UNSET)

        _loan = d.pop("loan", UNSET)
        loan: Union[Unset, BenefitDetailsLoan]
        if isinstance(_loan, Unset):
            loan = UNSET
        else:
            loan = BenefitDetailsLoan.from_dict(_loan)

        _car = d.pop("car", UNSET)
        car: Union[Unset, BenefitDetailsCar]
        if isinstance(_car, Unset):
            car = UNSET
        else:
            car = BenefitDetailsCar.from_dict(_car)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        id = d.pop("id", UNSET)

        benefit = cls(
            tax_year=tax_year,
            type=type,
            declaration_type=declaration_type,
            description=description,
            value=value,
            employee_contribution=employee_contribution,
            cash_equivalent=cash_equivalent,
            asset_type=asset_type,
            use_of_asset_type=use_of_asset_type,
            class_1a_type=class_1a_type,
            non_class_1a_type=non_class_1a_type,
            payment_type=payment_type,
            trading_organisation=trading_organisation,
            cash_equivalent_fuel=cash_equivalent_fuel,
            loan=loan,
            car=car,
            employee=employee,
            id=id,
        )

        return benefit



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="TieredPensionRate")


@attr.s(auto_attribs=True)
class TieredPensionRate:
    """Part of the TaxYearConfig that our engine uses to calculate tiered pension contributions.
    It is used internally when our engine performs calculations.
    You do not need to do anything with this model, it's provided purely for informational purposes.

        Attributes:
            name (Union[Unset, None, str]):
            description (Union[Unset, None, str]):
            range_start (Union[Unset, float]):
            rate (Union[Unset, float]):
    """

    name: Union[Unset, None, str] = UNSET
    description: Union[Unset, None, str] = UNSET
    range_start: Union[Unset, float] = UNSET
    rate: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        description = self.description
        range_start = self.range_start
        rate = self.rate

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if range_start is not UNSET:
            field_dict["rangeStart"] = range_start
        if rate is not UNSET:
            field_dict["rate"] = rate

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        range_start = d.pop("rangeStart", UNSET)

        rate = d.pop("rate", UNSET)

        tiered_pension_rate = cls(
            name=name,
            description=description,
            range_start=range_start,
            rate=rate,
        )

        return tiered_pension_rate



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="DirectDebitMandate")


@attr.s(auto_attribs=True)
class DirectDebitMandate:
    """
    Attributes:
        is_completed (Union[Unset, bool]):
        mandate_link (Union[Unset, None, str]):
        customer_link (Union[Unset, None, str]):
        bank_name (Union[Unset, None, str]):
        account_number_ending (Union[Unset, None, str]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    is_completed: Union[Unset, bool] = UNSET
    mandate_link: Union[Unset, None, str] = UNSET
    customer_link: Union[Unset, None, str] = UNSET
    bank_name: Union[Unset, None, str] = UNSET
    account_number_ending: Union[Unset, None, str] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        is_completed = self.is_completed
        mandate_link = self.mandate_link
        customer_link = self.customer_link
        bank_name = self.bank_name
        account_number_ending = self.account_number_ending
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if is_completed is not UNSET:
            field_dict["isCompleted"] = is_completed
        if mandate_link is not UNSET:
            field_dict["mandateLink"] = mandate_link
        if customer_link is not UNSET:
            field_dict["customerLink"] = customer_link
        if bank_name is not UNSET:
            field_dict["bankName"] = bank_name
        if account_number_ending is not UNSET:
            field_dict["accountNumberEnding"] = account_number_ending
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        is_completed = d.pop("isCompleted", UNSET)

        mandate_link = d.pop("mandateLink", UNSET)

        customer_link = d.pop("customerLink", UNSET)

        bank_name = d.pop("bankName", UNSET)

        account_number_ending = d.pop("accountNumberEnding", UNSET)

        id = d.pop("id", UNSET)

        direct_debit_mandate = cls(
            is_completed=is_completed,
            mandate_link=mandate_link,
            customer_link=customer_link,
            bank_name=bank_name,
            account_number_ending=account_number_ending,
            id=id,
        )

        return direct_debit_mandate



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsEmployeeFlexibleDrawdown")


@attr.s(auto_attribs=True)
class FpsEmployeeFlexibleDrawdown:
    """
    Attributes:
        flexibly_accessing_pension_rights (Union[Unset, None, str]):
        pension_death_benefit (Union[Unset, None, str]):
        taxable_payment (Union[Unset, None, str]):
        nontaxable_payment (Union[Unset, None, str]):
    """

    flexibly_accessing_pension_rights: Union[Unset, None, str] = UNSET
    pension_death_benefit: Union[Unset, None, str] = UNSET
    taxable_payment: Union[Unset, None, str] = UNSET
    nontaxable_payment: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        flexibly_accessing_pension_rights = self.flexibly_accessing_pension_rights
        pension_death_benefit = self.pension_death_benefit
        taxable_payment = self.taxable_payment
        nontaxable_payment = self.nontaxable_payment

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if flexibly_accessing_pension_rights is not UNSET:
            field_dict[
                "flexiblyAccessingPensionRights"
            ] = flexibly_accessing_pension_rights
        if pension_death_benefit is not UNSET:
            field_dict["pensionDeathBenefit"] = pension_death_benefit
        if taxable_payment is not UNSET:
            field_dict["taxablePayment"] = taxable_payment
        if nontaxable_payment is not UNSET:
            field_dict["nontaxablePayment"] = nontaxable_payment

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        flexibly_accessing_pension_rights = d.pop(
            "flexiblyAccessingPensionRights", UNSET
        )

        pension_death_benefit = d.pop("pensionDeathBenefit", UNSET)

        taxable_payment = d.pop("taxablePayment", UNSET)

        nontaxable_payment = d.pop("nontaxablePayment", UNSET)

        fps_employee_flexible_drawdown = cls(
            flexibly_accessing_pension_rights=flexibly_accessing_pension_rights,
            pension_death_benefit=pension_death_benefit,
            taxable_payment=taxable_payment,
            nontaxable_payment=nontaxable_payment,
        )

        return fps_employee_flexible_drawdown



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PaymentsCsvMappingColumn")


@attr.s(auto_attribs=True)
class PaymentsCsvMappingColumn:
    """
    Attributes:
        title (str):
        code (Union[Unset, None, str]):
        is_reference_only (Union[Unset, bool]): If set to true then no PayCode is mapped and this column is just for on-
            screen refernece only
        col_index (Union[Unset, int]):
        is_multiplier (Union[Unset, bool]):
        rate_col_index (Union[Unset, None, int]): If the mapping IsMultiplier, then this optionally specifies another
            column from which to get the value
        should_serialize_rate_col_index (Union[Unset, bool]):
    """

    title: str
    code: Union[Unset, None, str] = UNSET
    is_reference_only: Union[Unset, bool] = UNSET
    col_index: Union[Unset, int] = UNSET
    is_multiplier: Union[Unset, bool] = UNSET
    rate_col_index: Union[Unset, None, int] = UNSET
    should_serialize_rate_col_index: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        title = self.title
        code = self.code
        is_reference_only = self.is_reference_only
        col_index = self.col_index
        is_multiplier = self.is_multiplier
        rate_col_index = self.rate_col_index
        should_serialize_rate_col_index = self.should_serialize_rate_col_index

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "title": title,
            }
        )
        if code is not UNSET:
            field_dict["code"] = code
        if is_reference_only is not UNSET:
            field_dict["isReferenceOnly"] = is_reference_only
        if col_index is not UNSET:
            field_dict["colIndex"] = col_index
        if is_multiplier is not UNSET:
            field_dict["isMultiplier"] = is_multiplier
        if rate_col_index is not UNSET:
            field_dict["rateColIndex"] = rate_col_index
        if should_serialize_rate_col_index is not UNSET:
            field_dict["shouldSerializeRateColIndex"] = should_serialize_rate_col_index

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        title = d.pop("title")

        code = d.pop("code", UNSET)

        is_reference_only = d.pop("isReferenceOnly", UNSET)

        col_index = d.pop("colIndex", UNSET)

        is_multiplier = d.pop("isMultiplier", UNSET)

        rate_col_index = d.pop("rateColIndex", UNSET)

        should_serialize_rate_col_index = d.pop("shouldSerializeRateColIndex", UNSET)

        payments_csv_mapping_column = cls(
            title=title,
            code=code,
            is_reference_only=is_reference_only,
            col_index=col_index,
            is_multiplier=is_multiplier,
            rate_col_index=rate_col_index,
            should_serialize_rate_col_index=should_serialize_rate_col_index,
        )

        return payments_csv_mapping_column



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.ae_assessment import AeAssessment
from ..models.ae_employee_state import AeEmployeeState
from ..models.ae_exclusion_code import AeExclusionCode
from ..models.ae_uk_worker import AeUKWorker
from ..types import UNSET, Unset

T = TypeVar("T", bound="AutoEnrolment")


@attr.s(auto_attribs=True)
class AutoEnrolment:
    """
    Attributes:
        state (Union[Unset, AeEmployeeState]):
        state_date (Union[Unset, None, datetime.date]): [readonly]
        uk_worker (Union[Unset, AeUKWorker]):
        days_to_defer_assessment (Union[Unset, int]): The number of days, if any, to defer assessment of this employee.
            You're allowed to defer assessment of new employees by up to 3 months.
        postponement_date (Union[Unset, None, datetime.date]): [readonly] If a value is present, then employee wont be
            enrolled on an AE Pension until after this date.
            This is automatically set to a date if the employee is deemed to be an EligibleJobHolder, but the employer has
            set a Postponement period and this value is currently null
            This is automatically set to null if it already has a value and the employee is deemed NOT to be an
            EligibleJobHolder - ie, they previously qualified but now do not.
        defer_by_months_not_days (Union[Unset, bool]): If set to true then the value in DaysToDeferAssessment will be
            treated as a number of months, not a number of days
        exempt (Union[Unset, bool]):
        ae_exclusion_code (Union[Unset, AeExclusionCode]):
        ae_postponement_letter_sent (Union[Unset, bool]):
        last_assessment (Union[Unset, AeAssessment]): As part of AutoEnrolment we assess your Employees to see if they
            need to be auto-enroled in a Pension.
            This model shows the result of an assessment.
    """

    state: Union[Unset, AeEmployeeState] = UNSET
    state_date: Union[Unset, None, datetime.date] = UNSET
    uk_worker: Union[Unset, AeUKWorker] = UNSET
    days_to_defer_assessment: Union[Unset, int] = UNSET
    postponement_date: Union[Unset, None, datetime.date] = UNSET
    defer_by_months_not_days: Union[Unset, bool] = UNSET
    exempt: Union[Unset, bool] = UNSET
    ae_exclusion_code: Union[Unset, AeExclusionCode] = UNSET
    ae_postponement_letter_sent: Union[Unset, bool] = UNSET
    last_assessment: Union[Unset, AeAssessment] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.value

        state_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.state_date, Unset):
            state_date = self.state_date.isoformat() if self.state_date else None

        uk_worker: Union[Unset, str] = UNSET
        if not isinstance(self.uk_worker, Unset):
            uk_worker = self.uk_worker.value

        days_to_defer_assessment = self.days_to_defer_assessment
        postponement_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.postponement_date, Unset):
            postponement_date = (
                self.postponement_date.isoformat() if self.postponement_date else None
            )

        defer_by_months_not_days = self.defer_by_months_not_days
        exempt = self.exempt
        ae_exclusion_code: Union[Unset, str] = UNSET
        if not isinstance(self.ae_exclusion_code, Unset):
            ae_exclusion_code = self.ae_exclusion_code.value

        ae_postponement_letter_sent = self.ae_postponement_letter_sent
        last_assessment: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.last_assessment, Unset):
            last_assessment = self.last_assessment.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if state is not UNSET:
            field_dict["state"] = state
        if state_date is not UNSET:
            field_dict["stateDate"] = state_date
        if uk_worker is not UNSET:
            field_dict["ukWorker"] = uk_worker
        if days_to_defer_assessment is not UNSET:
            field_dict["daysToDeferAssessment"] = days_to_defer_assessment
        if postponement_date is not UNSET:
            field_dict["postponementDate"] = postponement_date
        if defer_by_months_not_days is not UNSET:
            field_dict["deferByMonthsNotDays"] = defer_by_months_not_days
        if exempt is not UNSET:
            field_dict["exempt"] = exempt
        if ae_exclusion_code is not UNSET:
            field_dict["aeExclusionCode"] = ae_exclusion_code
        if ae_postponement_letter_sent is not UNSET:
            field_dict["aePostponementLetterSent"] = ae_postponement_letter_sent
        if last_assessment is not UNSET:
            field_dict["lastAssessment"] = last_assessment

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _state = d.pop("state", UNSET)
        state: Union[Unset, AeEmployeeState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = AeEmployeeState(_state)

        _state_date = d.pop("stateDate", UNSET)
        state_date: Union[Unset, None, datetime.date]
        if _state_date is None:
            state_date = None
        elif isinstance(_state_date, Unset):
            state_date = UNSET
        else:
            state_date = isoparse(_state_date).date()

        _uk_worker = d.pop("ukWorker", UNSET)
        uk_worker: Union[Unset, AeUKWorker]
        if isinstance(_uk_worker, Unset):
            uk_worker = UNSET
        else:
            uk_worker = AeUKWorker(_uk_worker)

        days_to_defer_assessment = d.pop("daysToDeferAssessment", UNSET)

        _postponement_date = d.pop("postponementDate", UNSET)
        postponement_date: Union[Unset, None, datetime.date]
        if _postponement_date is None:
            postponement_date = None
        elif isinstance(_postponement_date, Unset):
            postponement_date = UNSET
        else:
            postponement_date = isoparse(_postponement_date).date()

        defer_by_months_not_days = d.pop("deferByMonthsNotDays", UNSET)

        exempt = d.pop("exempt", UNSET)

        _ae_exclusion_code = d.pop("aeExclusionCode", UNSET)
        ae_exclusion_code: Union[Unset, AeExclusionCode]
        if isinstance(_ae_exclusion_code, Unset):
            ae_exclusion_code = UNSET
        else:
            ae_exclusion_code = AeExclusionCode(_ae_exclusion_code)

        ae_postponement_letter_sent = d.pop("aePostponementLetterSent", UNSET)

        _last_assessment = d.pop("lastAssessment", UNSET)
        last_assessment: Union[Unset, AeAssessment]
        if isinstance(_last_assessment, Unset):
            last_assessment = UNSET
        else:
            last_assessment = AeAssessment.from_dict(_last_assessment)

        auto_enrolment = cls(
            state=state,
            state_date=state_date,
            uk_worker=uk_worker,
            days_to_defer_assessment=days_to_defer_assessment,
            postponement_date=postponement_date,
            defer_by_months_not_days=defer_by_months_not_days,
            exempt=exempt,
            ae_exclusion_code=ae_exclusion_code,
            ae_postponement_letter_sent=ae_postponement_letter_sent,
            last_assessment=last_assessment,
        )

        return auto_enrolment



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="NvrEmployment")


@attr.s(auto_attribs=True)
class NvrEmployment:
    """
    Attributes:
        pay_id (Union[Unset, None, str]):
    """

    pay_id: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        pay_id = self.pay_id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if pay_id is not UNSET:
            field_dict["payId"] = pay_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        pay_id = d.pop("payId", UNSET)

        nvr_employment = cls(
            pay_id=pay_id,
        )

        return nvr_employment



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsEmployeeNIlettersAndValues")


@attr.s(auto_attribs=True)
class FpsEmployeeNIlettersAndValues:
    """
    Attributes:
        n_iletter (Union[Unset, None, str]):
        gross_earnings_for_ni_cs_in_pd (Union[Unset, None, str]):
        gross_earnings_for_ni_cs_ytd (Union[Unset, None, str]):
        at_lelytd (Union[Unset, None, str]):
        le_lto_ptytd (Union[Unset, None, str]):
        p_tto_uelytd (Union[Unset, None, str]):
        total_emp_nic_in_pd (Union[Unset, None, str]):
        total_emp_nicytd (Union[Unset, None, str]):
        empee_contribns_in_pd (Union[Unset, None, str]):
        empee_contribns_ytd (Union[Unset, None, str]):
    """

    n_iletter: Union[Unset, None, str] = UNSET
    gross_earnings_for_ni_cs_in_pd: Union[Unset, None, str] = UNSET
    gross_earnings_for_ni_cs_ytd: Union[Unset, None, str] = UNSET
    at_lelytd: Union[Unset, None, str] = UNSET
    le_lto_ptytd: Union[Unset, None, str] = UNSET
    p_tto_uelytd: Union[Unset, None, str] = UNSET
    total_emp_nic_in_pd: Union[Unset, None, str] = UNSET
    total_emp_nicytd: Union[Unset, None, str] = UNSET
    empee_contribns_in_pd: Union[Unset, None, str] = UNSET
    empee_contribns_ytd: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        n_iletter = self.n_iletter
        gross_earnings_for_ni_cs_in_pd = self.gross_earnings_for_ni_cs_in_pd
        gross_earnings_for_ni_cs_ytd = self.gross_earnings_for_ni_cs_ytd
        at_lelytd = self.at_lelytd
        le_lto_ptytd = self.le_lto_ptytd
        p_tto_uelytd = self.p_tto_uelytd
        total_emp_nic_in_pd = self.total_emp_nic_in_pd
        total_emp_nicytd = self.total_emp_nicytd
        empee_contribns_in_pd = self.empee_contribns_in_pd
        empee_contribns_ytd = self.empee_contribns_ytd

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if n_iletter is not UNSET:
            field_dict["nIletter"] = n_iletter
        if gross_earnings_for_ni_cs_in_pd is not UNSET:
            field_dict["grossEarningsForNICsInPd"] = gross_earnings_for_ni_cs_in_pd
        if gross_earnings_for_ni_cs_ytd is not UNSET:
            field_dict["grossEarningsForNICsYTD"] = gross_earnings_for_ni_cs_ytd
        if at_lelytd is not UNSET:
            field_dict["atLELYTD"] = at_lelytd
        if le_lto_ptytd is not UNSET:
            field_dict["leLtoPTYTD"] = le_lto_ptytd
        if p_tto_uelytd is not UNSET:
            field_dict["pTtoUELYTD"] = p_tto_uelytd
        if total_emp_nic_in_pd is not UNSET:
            field_dict["totalEmpNICInPd"] = total_emp_nic_in_pd
        if total_emp_nicytd is not UNSET:
            field_dict["totalEmpNICYTD"] = total_emp_nicytd
        if empee_contribns_in_pd is not UNSET:
            field_dict["empeeContribnsInPd"] = empee_contribns_in_pd
        if empee_contribns_ytd is not UNSET:
            field_dict["empeeContribnsYTD"] = empee_contribns_ytd

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        n_iletter = d.pop("nIletter", UNSET)

        gross_earnings_for_ni_cs_in_pd = d.pop("grossEarningsForNICsInPd", UNSET)

        gross_earnings_for_ni_cs_ytd = d.pop("grossEarningsForNICsYTD", UNSET)

        at_lelytd = d.pop("atLELYTD", UNSET)

        le_lto_ptytd = d.pop("leLtoPTYTD", UNSET)

        p_tto_uelytd = d.pop("pTtoUELYTD", UNSET)

        total_emp_nic_in_pd = d.pop("totalEmpNICInPd", UNSET)

        total_emp_nicytd = d.pop("totalEmpNICYTD", UNSET)

        empee_contribns_in_pd = d.pop("empeeContribnsInPd", UNSET)

        empee_contribns_ytd = d.pop("empeeContribnsYTD", UNSET)

        fps_employee_n_iletters_and_values = cls(
            n_iletter=n_iletter,
            gross_earnings_for_ni_cs_in_pd=gross_earnings_for_ni_cs_in_pd,
            gross_earnings_for_ni_cs_ytd=gross_earnings_for_ni_cs_ytd,
            at_lelytd=at_lelytd,
            le_lto_ptytd=le_lto_ptytd,
            p_tto_uelytd=p_tto_uelytd,
            total_emp_nic_in_pd=total_emp_nic_in_pd,
            total_emp_nicytd=total_emp_nicytd,
            empee_contribns_in_pd=empee_contribns_in_pd,
            empee_contribns_ytd=empee_contribns_ytd,
        )

        return fps_employee_n_iletters_and_values



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.gender import Gender
from ..models.ni_letter_error import NiLetterError
from ..types import UNSET, Unset

T = TypeVar("T", bound="NiLetterValidationReportLine")


@attr.s(auto_attribs=True)
class NiLetterValidationReportLine:
    """
    Attributes:
        employee_id (Union[Unset, str]):
        payroll_code (Union[Unset, None, str]):
        name (Union[Unset, None, str]):
        ni_number (Union[Unset, None, str]):
        gender (Union[Unset, Gender]):
        date_of_birth (Union[Unset, datetime.date]):
        ni_letter_error (Union[Unset, NiLetterError]):
        error_details (Union[Unset, None, str]):
        ni_letter (Union[Unset, None, str]):
        suggested_letter (Union[Unset, None, str]):
    """

    employee_id: Union[Unset, str] = UNSET
    payroll_code: Union[Unset, None, str] = UNSET
    name: Union[Unset, None, str] = UNSET
    ni_number: Union[Unset, None, str] = UNSET
    gender: Union[Unset, Gender] = UNSET
    date_of_birth: Union[Unset, datetime.date] = UNSET
    ni_letter_error: Union[Unset, NiLetterError] = UNSET
    error_details: Union[Unset, None, str] = UNSET
    ni_letter: Union[Unset, None, str] = UNSET
    suggested_letter: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee_id = self.employee_id
        payroll_code = self.payroll_code
        name = self.name
        ni_number = self.ni_number
        gender: Union[Unset, str] = UNSET
        if not isinstance(self.gender, Unset):
            gender = self.gender.value

        date_of_birth: Union[Unset, str] = UNSET
        if not isinstance(self.date_of_birth, Unset):
            date_of_birth = self.date_of_birth.isoformat()

        ni_letter_error: Union[Unset, str] = UNSET
        if not isinstance(self.ni_letter_error, Unset):
            ni_letter_error = self.ni_letter_error.value

        error_details = self.error_details
        ni_letter = self.ni_letter
        suggested_letter = self.suggested_letter

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee_id is not UNSET:
            field_dict["employeeId"] = employee_id
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if name is not UNSET:
            field_dict["name"] = name
        if ni_number is not UNSET:
            field_dict["niNumber"] = ni_number
        if gender is not UNSET:
            field_dict["gender"] = gender
        if date_of_birth is not UNSET:
            field_dict["dateOfBirth"] = date_of_birth
        if ni_letter_error is not UNSET:
            field_dict["niLetterError"] = ni_letter_error
        if error_details is not UNSET:
            field_dict["errorDetails"] = error_details
        if ni_letter is not UNSET:
            field_dict["niLetter"] = ni_letter
        if suggested_letter is not UNSET:
            field_dict["suggestedLetter"] = suggested_letter

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employee_id = d.pop("employeeId", UNSET)

        payroll_code = d.pop("payrollCode", UNSET)

        name = d.pop("name", UNSET)

        ni_number = d.pop("niNumber", UNSET)

        _gender = d.pop("gender", UNSET)
        gender: Union[Unset, Gender]
        if isinstance(_gender, Unset):
            gender = UNSET
        else:
            gender = Gender(_gender)

        _date_of_birth = d.pop("dateOfBirth", UNSET)
        date_of_birth: Union[Unset, datetime.date]
        if isinstance(_date_of_birth, Unset):
            date_of_birth = UNSET
        else:
            date_of_birth = isoparse(_date_of_birth).date()

        _ni_letter_error = d.pop("niLetterError", UNSET)
        ni_letter_error: Union[Unset, NiLetterError]
        if isinstance(_ni_letter_error, Unset):
            ni_letter_error = UNSET
        else:
            ni_letter_error = NiLetterError(_ni_letter_error)

        error_details = d.pop("errorDetails", UNSET)

        ni_letter = d.pop("niLetter", UNSET)

        suggested_letter = d.pop("suggestedLetter", UNSET)

        ni_letter_validation_report_line = cls(
            employee_id=employee_id,
            payroll_code=payroll_code,
            name=name,
            ni_number=ni_number,
            gender=gender,
            date_of_birth=date_of_birth,
            ni_letter_error=ni_letter_error,
            error_details=error_details,
            ni_letter=ni_letter,
            suggested_letter=suggested_letter,
        )

        return ni_letter_validation_report_line



from enum import Enum


class PensionRule(str, Enum):
    RELIEFATSOURCE = "ReliefAtSource"
    SALARYSACRIFICE = "SalarySacrifice"
    NETPAYARRANGEMENT = "NetPayArrangement"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.papdis_document import PapdisDocument
from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisDocumentReportResponse")


@attr.s(auto_attribs=True)
class PapdisDocumentReportResponse:
    """Used to encapsulate a response for any of the reports.
    See the Introduction Guide for Reports for more details

        Attributes:
            type (Union[Unset, None, str]): [readonly] The content-type, this would usually be the same as the accept header
                you provided when you requested the report
            content (Union[Unset, None, str]): [readonly] This could contain a link to a PDF file, HTML content or other
                content, depending on the Type value.
            model (Union[Unset, PapdisDocument]): PAPDIS stands for 'Payroll and Pension Data Interface Standard'.
                It is an industry standard for exchanging data between payroll software and pension providers.
                Our system provides an export of data in this standard and these models are used to represent the data.
                Whilst the relevant reporting API endpoint can provide this data as a JSON entity, it is usually represented in
                CSV or XML format which our API also provides.
    """

    type: Union[Unset, None, str] = UNSET
    content: Union[Unset, None, str] = UNSET
    model: Union[Unset, PapdisDocument] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type = self.type
        content = self.content
        model: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.model, Unset):
            model = self.model.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if content is not UNSET:
            field_dict["content"] = content
        if model is not UNSET:
            field_dict["model"] = model

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type", UNSET)

        content = d.pop("content", UNSET)

        _model = d.pop("model", UNSET)
        model: Union[Unset, PapdisDocument]
        if isinstance(_model, Unset):
            model = UNSET
        else:
            model = PapdisDocument.from_dict(_model)

        papdis_document_report_response = cls(
            type=type,
            content=content,
            model=model,
        )

        return papdis_document_report_response



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.gov_talk_error import GovTalkError
from ..models.submission_status import SubmissionStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="GovTalkSubmission")


@attr.s(auto_attribs=True)
class GovTalkSubmission:
    """
    Attributes:
        raw_response (Union[Unset, None, str]):
        message_class (Union[Unset, None, str]):
        poll_interval (Union[Unset, int]):
        last_poll (Union[Unset, None, datetime.datetime]):
        correlation_id (Union[Unset, None, str]):
        url (Union[Unset, None, str]):
        status (Union[Unset, SubmissionStatus]):
        message (Union[Unset, None, str]):
        i_rmark (Union[Unset, None, str]):
        errors_json (Union[Unset, None, str]):
        errors (Union[Unset, None, List[GovTalkError]]):
        submitted_at (Union[Unset, None, datetime.datetime]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    raw_response: Union[Unset, None, str] = UNSET
    message_class: Union[Unset, None, str] = UNSET
    poll_interval: Union[Unset, int] = UNSET
    last_poll: Union[Unset, None, datetime.datetime] = UNSET
    correlation_id: Union[Unset, None, str] = UNSET
    url: Union[Unset, None, str] = UNSET
    status: Union[Unset, SubmissionStatus] = UNSET
    message: Union[Unset, None, str] = UNSET
    i_rmark: Union[Unset, None, str] = UNSET
    errors_json: Union[Unset, None, str] = UNSET
    errors: Union[Unset, None, List[GovTalkError]] = UNSET
    submitted_at: Union[Unset, None, datetime.datetime] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        raw_response = self.raw_response
        message_class = self.message_class
        poll_interval = self.poll_interval
        last_poll: Union[Unset, None, str] = UNSET
        if not isinstance(self.last_poll, Unset):
            last_poll = self.last_poll.isoformat() if self.last_poll else None

        correlation_id = self.correlation_id
        url = self.url
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        message = self.message
        i_rmark = self.i_rmark
        errors_json = self.errors_json
        errors: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.errors, Unset):
            if self.errors is None:
                errors = None
            else:
                errors = []
                for errors_item_data in self.errors:
                    errors_item = errors_item_data.to_dict()

                    errors.append(errors_item)

        submitted_at: Union[Unset, None, str] = UNSET
        if not isinstance(self.submitted_at, Unset):
            submitted_at = self.submitted_at.isoformat() if self.submitted_at else None

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if raw_response is not UNSET:
            field_dict["rawResponse"] = raw_response
        if message_class is not UNSET:
            field_dict["messageClass"] = message_class
        if poll_interval is not UNSET:
            field_dict["pollInterval"] = poll_interval
        if last_poll is not UNSET:
            field_dict["lastPoll"] = last_poll
        if correlation_id is not UNSET:
            field_dict["correlationId"] = correlation_id
        if url is not UNSET:
            field_dict["url"] = url
        if status is not UNSET:
            field_dict["status"] = status
        if message is not UNSET:
            field_dict["message"] = message
        if i_rmark is not UNSET:
            field_dict["iRmark"] = i_rmark
        if errors_json is not UNSET:
            field_dict["errorsJson"] = errors_json
        if errors is not UNSET:
            field_dict["errors"] = errors
        if submitted_at is not UNSET:
            field_dict["submittedAt"] = submitted_at
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        raw_response = d.pop("rawResponse", UNSET)

        message_class = d.pop("messageClass", UNSET)

        poll_interval = d.pop("pollInterval", UNSET)

        _last_poll = d.pop("lastPoll", UNSET)
        last_poll: Union[Unset, None, datetime.datetime]
        if _last_poll is None:
            last_poll = None
        elif isinstance(_last_poll, Unset):
            last_poll = UNSET
        else:
            last_poll = isoparse(_last_poll)

        correlation_id = d.pop("correlationId", UNSET)

        url = d.pop("url", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, SubmissionStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = SubmissionStatus(_status)

        message = d.pop("message", UNSET)

        i_rmark = d.pop("iRmark", UNSET)

        errors_json = d.pop("errorsJson", UNSET)

        errors = []
        _errors = d.pop("errors", UNSET)
        for errors_item_data in _errors or []:
            errors_item = GovTalkError.from_dict(errors_item_data)

            errors.append(errors_item)

        _submitted_at = d.pop("submittedAt", UNSET)
        submitted_at: Union[Unset, None, datetime.datetime]
        if _submitted_at is None:
            submitted_at = None
        elif isinstance(_submitted_at, Unset):
            submitted_at = UNSET
        else:
            submitted_at = isoparse(_submitted_at)

        id = d.pop("id", UNSET)

        gov_talk_submission = cls(
            raw_response=raw_response,
            message_class=message_class,
            poll_interval=poll_interval,
            last_poll=last_poll,
            correlation_id=correlation_id,
            url=url,
            status=status,
            message=message,
            i_rmark=i_rmark,
            errors_json=errors_json,
            errors=errors,
            submitted_at=submitted_at,
            id=id,
        )

        return gov_talk_submission



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.cis_details import CisDetails
from ..models.department import Department
from ..models.directorship_details import DirectorshipDetails
from ..models.furlough_calculation_basis import FurloughCalculationBasis
from ..models.item import Item
from ..models.leaver_details import LeaverDetails
from ..models.starter_details import StarterDetails
from ..models.veteran_details import VeteranDetails
from ..types import UNSET, Unset

T = TypeVar("T", bound="EmploymentDetails")


@attr.s(auto_attribs=True)
class EmploymentDetails:
    """
    Attributes:
        payroll_code (str): The Employees Payroll Code. Must be unique within the Employer.
        cis_sub_contractor (Union[Unset, bool]): Set to True if this Employee is a CIS Subcontractor. The
            <code>Cis</code> property contains further information
        job_title (Union[Unset, None, str]): Job Title of Primary post of the Employee
        on_hold (Union[Unset, bool]): Set to true to temporarily exclude the employee from payruns
        on_furlough (Union[Unset, bool]): Set to true if the employee is on furlough.
        furlough_start (Union[Unset, None, datetime.date]): Furlough Start Date.
        furlough_end (Union[Unset, None, datetime.date]): Furlough End Date.
        furlough_calculation_basis (Union[Unset, FurloughCalculationBasis]):
        furlough_calculation_basis_amount (Union[Unset, float]):
        partial_furlough (Union[Unset, bool]): Set to true if the employee is partially furloughed.
        furlough_hours_normally_worked (Union[Unset, float]):
        furlough_hours_on_furlough (Union[Unset, float]):
        is_apprentice (Union[Unset, bool]): Set to True if this Employee is an apprentice. This affects the calculations
            for National Minimum Wage
        apprenticeship_start_date (Union[Unset, None, datetime.date]):
        apprenticeship_end_date (Union[Unset, None, datetime.date]):
        working_pattern (Union[Unset, None, str]): Used when calculating payments for Leave.
            If null then the default Working Pattern is used
        force_previous_payroll_code (Union[Unset, None, str]): If this property has a non-empty value then a change of
            Payroll code will be declared on the next FPS.
        starter_details (Union[Unset, StarterDetails]):
        directorship_details (Union[Unset, DirectorshipDetails]):
        leaver_details (Union[Unset, LeaverDetails]):
        cis (Union[Unset, CisDetails]): If an Employee is marked as a CIS Subcontractor then this model provides further
            details specifically related to the CIS details of the Employee.
        department (Union[Unset, Department]):
        posts (Union[Unset, None, List[Item]]): List of Posts held by Employee
        is_working_in_free_port (Union[Unset, bool]): Flag indicating the employee is employed in a Freeport
        veteran_details (Union[Unset, VeteranDetails]): Employment details for veterans
    """

    payroll_code: str
    cis_sub_contractor: Union[Unset, bool] = UNSET
    job_title: Union[Unset, None, str] = UNSET
    on_hold: Union[Unset, bool] = UNSET
    on_furlough: Union[Unset, bool] = UNSET
    furlough_start: Union[Unset, None, datetime.date] = UNSET
    furlough_end: Union[Unset, None, datetime.date] = UNSET
    furlough_calculation_basis: Union[Unset, FurloughCalculationBasis] = UNSET
    furlough_calculation_basis_amount: Union[Unset, float] = UNSET
    partial_furlough: Union[Unset, bool] = UNSET
    furlough_hours_normally_worked: Union[Unset, float] = UNSET
    furlough_hours_on_furlough: Union[Unset, float] = UNSET
    is_apprentice: Union[Unset, bool] = UNSET
    apprenticeship_start_date: Union[Unset, None, datetime.date] = UNSET
    apprenticeship_end_date: Union[Unset, None, datetime.date] = UNSET
    working_pattern: Union[Unset, None, str] = UNSET
    force_previous_payroll_code: Union[Unset, None, str] = UNSET
    starter_details: Union[Unset, StarterDetails] = UNSET
    directorship_details: Union[Unset, DirectorshipDetails] = UNSET
    leaver_details: Union[Unset, LeaverDetails] = UNSET
    cis: Union[Unset, CisDetails] = UNSET
    department: Union[Unset, Department] = UNSET
    posts: Union[Unset, None, List[Item]] = UNSET
    is_working_in_free_port: Union[Unset, bool] = UNSET
    veteran_details: Union[Unset, VeteranDetails] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        payroll_code = self.payroll_code
        cis_sub_contractor = self.cis_sub_contractor
        job_title = self.job_title
        on_hold = self.on_hold
        on_furlough = self.on_furlough
        furlough_start: Union[Unset, None, str] = UNSET
        if not isinstance(self.furlough_start, Unset):
            furlough_start = (
                self.furlough_start.isoformat() if self.furlough_start else None
            )

        furlough_end: Union[Unset, None, str] = UNSET
        if not isinstance(self.furlough_end, Unset):
            furlough_end = self.furlough_end.isoformat() if self.furlough_end else None

        furlough_calculation_basis: Union[Unset, str] = UNSET
        if not isinstance(self.furlough_calculation_basis, Unset):
            furlough_calculation_basis = self.furlough_calculation_basis.value

        furlough_calculation_basis_amount = self.furlough_calculation_basis_amount
        partial_furlough = self.partial_furlough
        furlough_hours_normally_worked = self.furlough_hours_normally_worked
        furlough_hours_on_furlough = self.furlough_hours_on_furlough
        is_apprentice = self.is_apprentice
        apprenticeship_start_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.apprenticeship_start_date, Unset):
            apprenticeship_start_date = (
                self.apprenticeship_start_date.isoformat()
                if self.apprenticeship_start_date
                else None
            )

        apprenticeship_end_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.apprenticeship_end_date, Unset):
            apprenticeship_end_date = (
                self.apprenticeship_end_date.isoformat()
                if self.apprenticeship_end_date
                else None
            )

        working_pattern = self.working_pattern
        force_previous_payroll_code = self.force_previous_payroll_code
        starter_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.starter_details, Unset):
            starter_details = self.starter_details.to_dict()

        directorship_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.directorship_details, Unset):
            directorship_details = self.directorship_details.to_dict()

        leaver_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.leaver_details, Unset):
            leaver_details = self.leaver_details.to_dict()

        cis: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.cis, Unset):
            cis = self.cis.to_dict()

        department: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.department, Unset):
            department = self.department.to_dict()

        posts: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.posts, Unset):
            if self.posts is None:
                posts = None
            else:
                posts = []
                for posts_item_data in self.posts:
                    posts_item = posts_item_data.to_dict()

                    posts.append(posts_item)

        is_working_in_free_port = self.is_working_in_free_port
        veteran_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.veteran_details, Unset):
            veteran_details = self.veteran_details.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "payrollCode": payroll_code,
            }
        )
        if cis_sub_contractor is not UNSET:
            field_dict["cisSubContractor"] = cis_sub_contractor
        if job_title is not UNSET:
            field_dict["jobTitle"] = job_title
        if on_hold is not UNSET:
            field_dict["onHold"] = on_hold
        if on_furlough is not UNSET:
            field_dict["onFurlough"] = on_furlough
        if furlough_start is not UNSET:
            field_dict["furloughStart"] = furlough_start
        if furlough_end is not UNSET:
            field_dict["furloughEnd"] = furlough_end
        if furlough_calculation_basis is not UNSET:
            field_dict["furloughCalculationBasis"] = furlough_calculation_basis
        if furlough_calculation_basis_amount is not UNSET:
            field_dict[
                "furloughCalculationBasisAmount"
            ] = furlough_calculation_basis_amount
        if partial_furlough is not UNSET:
            field_dict["partialFurlough"] = partial_furlough
        if furlough_hours_normally_worked is not UNSET:
            field_dict["furloughHoursNormallyWorked"] = furlough_hours_normally_worked
        if furlough_hours_on_furlough is not UNSET:
            field_dict["furloughHoursOnFurlough"] = furlough_hours_on_furlough
        if is_apprentice is not UNSET:
            field_dict["isApprentice"] = is_apprentice
        if apprenticeship_start_date is not UNSET:
            field_dict["apprenticeshipStartDate"] = apprenticeship_start_date
        if apprenticeship_end_date is not UNSET:
            field_dict["apprenticeshipEndDate"] = apprenticeship_end_date
        if working_pattern is not UNSET:
            field_dict["workingPattern"] = working_pattern
        if force_previous_payroll_code is not UNSET:
            field_dict["forcePreviousPayrollCode"] = force_previous_payroll_code
        if starter_details is not UNSET:
            field_dict["starterDetails"] = starter_details
        if directorship_details is not UNSET:
            field_dict["directorshipDetails"] = directorship_details
        if leaver_details is not UNSET:
            field_dict["leaverDetails"] = leaver_details
        if cis is not UNSET:
            field_dict["cis"] = cis
        if department is not UNSET:
            field_dict["department"] = department
        if posts is not UNSET:
            field_dict["posts"] = posts
        if is_working_in_free_port is not UNSET:
            field_dict["isWorkingInFreePort"] = is_working_in_free_port
        if veteran_details is not UNSET:
            field_dict["veteranDetails"] = veteran_details

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        payroll_code = d.pop("payrollCode")

        cis_sub_contractor = d.pop("cisSubContractor", UNSET)

        job_title = d.pop("jobTitle", UNSET)

        on_hold = d.pop("onHold", UNSET)

        on_furlough = d.pop("onFurlough", UNSET)

        _furlough_start = d.pop("furloughStart", UNSET)
        furlough_start: Union[Unset, None, datetime.date]
        if _furlough_start is None:
            furlough_start = None
        elif isinstance(_furlough_start, Unset):
            furlough_start = UNSET
        else:
            furlough_start = isoparse(_furlough_start).date()

        _furlough_end = d.pop("furloughEnd", UNSET)
        furlough_end: Union[Unset, None, datetime.date]
        if _furlough_end is None:
            furlough_end = None
        elif isinstance(_furlough_end, Unset):
            furlough_end = UNSET
        else:
            furlough_end = isoparse(_furlough_end).date()

        _furlough_calculation_basis = d.pop("furloughCalculationBasis", UNSET)
        furlough_calculation_basis: Union[Unset, FurloughCalculationBasis]
        if isinstance(_furlough_calculation_basis, Unset):
            furlough_calculation_basis = UNSET
        else:
            furlough_calculation_basis = FurloughCalculationBasis(
                _furlough_calculation_basis
            )

        furlough_calculation_basis_amount = d.pop(
            "furloughCalculationBasisAmount", UNSET
        )

        partial_furlough = d.pop("partialFurlough", UNSET)

        furlough_hours_normally_worked = d.pop("furloughHoursNormallyWorked", UNSET)

        furlough_hours_on_furlough = d.pop("furloughHoursOnFurlough", UNSET)

        is_apprentice = d.pop("isApprentice", UNSET)

        _apprenticeship_start_date = d.pop("apprenticeshipStartDate", UNSET)
        apprenticeship_start_date: Union[Unset, None, datetime.date]
        if _apprenticeship_start_date is None:
            apprenticeship_start_date = None
        elif isinstance(_apprenticeship_start_date, Unset):
            apprenticeship_start_date = UNSET
        else:
            apprenticeship_start_date = isoparse(_apprenticeship_start_date).date()

        _apprenticeship_end_date = d.pop("apprenticeshipEndDate", UNSET)
        apprenticeship_end_date: Union[Unset, None, datetime.date]
        if _apprenticeship_end_date is None:
            apprenticeship_end_date = None
        elif isinstance(_apprenticeship_end_date, Unset):
            apprenticeship_end_date = UNSET
        else:
            apprenticeship_end_date = isoparse(_apprenticeship_end_date).date()

        working_pattern = d.pop("workingPattern", UNSET)

        force_previous_payroll_code = d.pop("forcePreviousPayrollCode", UNSET)

        _starter_details = d.pop("starterDetails", UNSET)
        starter_details: Union[Unset, StarterDetails]
        if isinstance(_starter_details, Unset):
            starter_details = UNSET
        else:
            starter_details = StarterDetails.from_dict(_starter_details)

        _directorship_details = d.pop("directorshipDetails", UNSET)
        directorship_details: Union[Unset, DirectorshipDetails]
        if isinstance(_directorship_details, Unset):
            directorship_details = UNSET
        else:
            directorship_details = DirectorshipDetails.from_dict(_directorship_details)

        _leaver_details = d.pop("leaverDetails", UNSET)
        leaver_details: Union[Unset, LeaverDetails]
        if isinstance(_leaver_details, Unset):
            leaver_details = UNSET
        else:
            leaver_details = LeaverDetails.from_dict(_leaver_details)

        _cis = d.pop("cis", UNSET)
        cis: Union[Unset, CisDetails]
        if isinstance(_cis, Unset):
            cis = UNSET
        else:
            cis = CisDetails.from_dict(_cis)

        _department = d.pop("department", UNSET)
        department: Union[Unset, Department]
        if isinstance(_department, Unset):
            department = UNSET
        else:
            department = Department.from_dict(_department)

        posts = []
        _posts = d.pop("posts", UNSET)
        for posts_item_data in _posts or []:
            posts_item = Item.from_dict(posts_item_data)

            posts.append(posts_item)

        is_working_in_free_port = d.pop("isWorkingInFreePort", UNSET)

        _veteran_details = d.pop("veteranDetails", UNSET)
        veteran_details: Union[Unset, VeteranDetails]
        if isinstance(_veteran_details, Unset):
            veteran_details = UNSET
        else:
            veteran_details = VeteranDetails.from_dict(_veteran_details)

        employment_details = cls(
            payroll_code=payroll_code,
            cis_sub_contractor=cis_sub_contractor,
            job_title=job_title,
            on_hold=on_hold,
            on_furlough=on_furlough,
            furlough_start=furlough_start,
            furlough_end=furlough_end,
            furlough_calculation_basis=furlough_calculation_basis,
            furlough_calculation_basis_amount=furlough_calculation_basis_amount,
            partial_furlough=partial_furlough,
            furlough_hours_normally_worked=furlough_hours_normally_worked,
            furlough_hours_on_furlough=furlough_hours_on_furlough,
            is_apprentice=is_apprentice,
            apprenticeship_start_date=apprenticeship_start_date,
            apprenticeship_end_date=apprenticeship_end_date,
            working_pattern=working_pattern,
            force_previous_payroll_code=force_previous_payroll_code,
            starter_details=starter_details,
            directorship_details=directorship_details,
            leaver_details=leaver_details,
            cis=cis,
            department=department,
            posts=posts,
            is_working_in_free_port=is_working_in_free_port,
            veteran_details=veteran_details,
        )

        return employment_details



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.pension_contribution_level_type import PensionContributionLevelType
from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkerGroup")


@attr.s(auto_attribs=True)
class WorkerGroup:
    """
    Attributes:
        name (str):
        contribution_level_type (Union[Unset, PensionContributionLevelType]):
        employee_contribution (Union[Unset, float]):
        employee_contribution_is_percentage (Union[Unset, bool]):
        employer_contribution (Union[Unset, float]):
        employer_contribution_is_percentage (Union[Unset, bool]):
        employer_contribution_top_up_percentage (Union[Unset, float]): Increase Employer Contribution by this percentage
            of the Employee Contribution
        custom_threshold (Union[Unset, bool]):
        lower_limit (Union[Unset, float]):
        upper_limit (Union[Unset, float]):
        papdis_group (Union[Unset, None, str]):
        papdis_sub_group (Union[Unset, None, str]):
        local_authority_number (Union[Unset, None, str]): Only applicable if ContributionLevelType is Tp2020
        school_employer_type (Union[Unset, None, str]): Only applicable if ContributionLevelType is Tp2020
        worker_group_id (Union[Unset, str]): [readonly]
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    name: str
    contribution_level_type: Union[Unset, PensionContributionLevelType] = UNSET
    employee_contribution: Union[Unset, float] = UNSET
    employee_contribution_is_percentage: Union[Unset, bool] = UNSET
    employer_contribution: Union[Unset, float] = UNSET
    employer_contribution_is_percentage: Union[Unset, bool] = UNSET
    employer_contribution_top_up_percentage: Union[Unset, float] = UNSET
    custom_threshold: Union[Unset, bool] = UNSET
    lower_limit: Union[Unset, float] = UNSET
    upper_limit: Union[Unset, float] = UNSET
    papdis_group: Union[Unset, None, str] = UNSET
    papdis_sub_group: Union[Unset, None, str] = UNSET
    local_authority_number: Union[Unset, None, str] = UNSET
    school_employer_type: Union[Unset, None, str] = UNSET
    worker_group_id: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        contribution_level_type: Union[Unset, str] = UNSET
        if not isinstance(self.contribution_level_type, Unset):
            contribution_level_type = self.contribution_level_type.value

        employee_contribution = self.employee_contribution
        employee_contribution_is_percentage = self.employee_contribution_is_percentage
        employer_contribution = self.employer_contribution
        employer_contribution_is_percentage = self.employer_contribution_is_percentage
        employer_contribution_top_up_percentage = (
            self.employer_contribution_top_up_percentage
        )
        custom_threshold = self.custom_threshold
        lower_limit = self.lower_limit
        upper_limit = self.upper_limit
        papdis_group = self.papdis_group
        papdis_sub_group = self.papdis_sub_group
        local_authority_number = self.local_authority_number
        school_employer_type = self.school_employer_type
        worker_group_id = self.worker_group_id
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )
        if contribution_level_type is not UNSET:
            field_dict["contributionLevelType"] = contribution_level_type
        if employee_contribution is not UNSET:
            field_dict["employeeContribution"] = employee_contribution
        if employee_contribution_is_percentage is not UNSET:
            field_dict[
                "employeeContributionIsPercentage"
            ] = employee_contribution_is_percentage
        if employer_contribution is not UNSET:
            field_dict["employerContribution"] = employer_contribution
        if employer_contribution_is_percentage is not UNSET:
            field_dict[
                "employerContributionIsPercentage"
            ] = employer_contribution_is_percentage
        if employer_contribution_top_up_percentage is not UNSET:
            field_dict[
                "employerContributionTopUpPercentage"
            ] = employer_contribution_top_up_percentage
        if custom_threshold is not UNSET:
            field_dict["customThreshold"] = custom_threshold
        if lower_limit is not UNSET:
            field_dict["lowerLimit"] = lower_limit
        if upper_limit is not UNSET:
            field_dict["upperLimit"] = upper_limit
        if papdis_group is not UNSET:
            field_dict["papdisGroup"] = papdis_group
        if papdis_sub_group is not UNSET:
            field_dict["papdisSubGroup"] = papdis_sub_group
        if local_authority_number is not UNSET:
            field_dict["localAuthorityNumber"] = local_authority_number
        if school_employer_type is not UNSET:
            field_dict["schoolEmployerType"] = school_employer_type
        if worker_group_id is not UNSET:
            field_dict["workerGroupId"] = worker_group_id
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        _contribution_level_type = d.pop("contributionLevelType", UNSET)
        contribution_level_type: Union[Unset, PensionContributionLevelType]
        if isinstance(_contribution_level_type, Unset):
            contribution_level_type = UNSET
        else:
            contribution_level_type = PensionContributionLevelType(
                _contribution_level_type
            )

        employee_contribution = d.pop("employeeContribution", UNSET)

        employee_contribution_is_percentage = d.pop(
            "employeeContributionIsPercentage", UNSET
        )

        employer_contribution = d.pop("employerContribution", UNSET)

        employer_contribution_is_percentage = d.pop(
            "employerContributionIsPercentage", UNSET
        )

        employer_contribution_top_up_percentage = d.pop(
            "employerContributionTopUpPercentage", UNSET
        )

        custom_threshold = d.pop("customThreshold", UNSET)

        lower_limit = d.pop("lowerLimit", UNSET)

        upper_limit = d.pop("upperLimit", UNSET)

        papdis_group = d.pop("papdisGroup", UNSET)

        papdis_sub_group = d.pop("papdisSubGroup", UNSET)

        local_authority_number = d.pop("localAuthorityNumber", UNSET)

        school_employer_type = d.pop("schoolEmployerType", UNSET)

        worker_group_id = d.pop("workerGroupId", UNSET)

        id = d.pop("id", UNSET)

        worker_group = cls(
            name=name,
            contribution_level_type=contribution_level_type,
            employee_contribution=employee_contribution,
            employee_contribution_is_percentage=employee_contribution_is_percentage,
            employer_contribution=employer_contribution,
            employer_contribution_is_percentage=employer_contribution_is_percentage,
            employer_contribution_top_up_percentage=employer_contribution_top_up_percentage,
            custom_threshold=custom_threshold,
            lower_limit=lower_limit,
            upper_limit=upper_limit,
            papdis_group=papdis_group,
            papdis_sub_group=papdis_sub_group,
            local_authority_number=local_authority_number,
            school_employer_type=school_employer_type,
            worker_group_id=worker_group_id,
            id=id,
        )

        return worker_group



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EpsAccount")


@attr.s(auto_attribs=True)
class EpsAccount:
    """Used on an EPS to send bank account information

    Attributes:
        account_holders_name (Union[Unset, None, str]):
        account_no (Union[Unset, None, str]):
        sort_code (Union[Unset, None, str]):
        building_soc_ref (Union[Unset, None, str]):
    """

    account_holders_name: Union[Unset, None, str] = UNSET
    account_no: Union[Unset, None, str] = UNSET
    sort_code: Union[Unset, None, str] = UNSET
    building_soc_ref: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        account_holders_name = self.account_holders_name
        account_no = self.account_no
        sort_code = self.sort_code
        building_soc_ref = self.building_soc_ref

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if account_holders_name is not UNSET:
            field_dict["accountHoldersName"] = account_holders_name
        if account_no is not UNSET:
            field_dict["accountNo"] = account_no
        if sort_code is not UNSET:
            field_dict["sortCode"] = sort_code
        if building_soc_ref is not UNSET:
            field_dict["buildingSocRef"] = building_soc_ref

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        account_holders_name = d.pop("accountHoldersName", UNSET)

        account_no = d.pop("accountNo", UNSET)

        sort_code = d.pop("sortCode", UNSET)

        building_soc_ref = d.pop("buildingSocRef", UNSET)

        eps_account = cls(
            account_holders_name=account_holders_name,
            account_no=account_no,
            sort_code=sort_code,
            building_soc_ref=building_soc_ref,
        )

        return eps_account
